package service

import (
	cDto "Medical_OAuth/common/dto"
	"Medical_OAuth/common/global"
	"errors"
	"fmt"
	"gogs.baozhida.cn/zoie/OAuth-core/service"

	"gorm.io/gorm"

	"Medical_OAuth/app/admin/model"
	"Medical_OAuth/app/admin/service/dto"
)

type SysDictTypeService struct {
	service.Service
}

// GetPage 获取列表
func (e *SysDictTypeService) GetPage(c *dto.SysDictTypeGetPageReq, list *[]model.SysDictType, count *int64) error {
	var err error
	var data model.SysDictType

	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
		).
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}
	return nil
}

// Get 获取对象
func (e *SysDictTypeService) Get(d *dto.SysDictTypeGetReq, model *model.SysDictType) error {
	err := e.Orm.First(model, d.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.GetNotFoundOrNoPermissionErr
		}
		return global.GetFailedErr
	}
	return nil
}

// Insert 创建对象
func (e *SysDictTypeService) Insert(c *dto.SysDictTypeInsertReq) error {
	var err error
	var data model.SysDictType
	c.Generate(&data)
	var count int64
	e.Orm.Model(&data).Where("dict_type = ?", data.DictType).Count(&count)
	if count > 0 {
		return errors.New(fmt.Sprintf("当前字典类型[%s]已经存在！", data.DictType))
	}
	err = e.Orm.Create(&data).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}
	return nil
}

// Update 修改对象
func (e *SysDictTypeService) Update(c *dto.SysDictTypeUpdateReq) error {
	var err error
	var dictTypeModel = model.SysDictType{}
	e.Orm.First(&dictTypeModel, c.GetId())
	c.Generate(&dictTypeModel)
	db := e.Orm.Save(&dictTypeModel)
	if db.Error != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}
	if db.RowsAffected == 0 {
		return global.UpdateNotFoundOrNoPermissionErr
	}
	return nil
}

// Remove 删除
func (e *SysDictTypeService) Remove(d *dto.SysDictTypeDeleteReq) error {
	var err error
	var data model.SysDictType

	db := e.Orm.Delete(&data, d.GetId())
	err = db.Error
	if err != nil {
		e.Log.Errorf("Delete error: %s", err)
		return global.DeleteFailedErr
	}
	if db.RowsAffected == 0 {
		return global.DeleteNotFoundOrNoPermissionErr
	}
	return nil
}

// GetAll 获取所有
func (e *SysDictTypeService) GetAll(c *dto.SysDictTypeGetPageReq, list *[]model.SysDictType) error {
	var err error
	var data model.SysDictType

	err = e.Orm.Model(&data).
		Scopes(cDto.MakeCondition(c.GetNeedSearch())).
		Find(list).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.GetFailedErr
	}
	return nil
}
