package service

import (
	"Medical_OAuth/common/actions"
	cDto "Medical_OAuth/common/dto"
	"Medical_OAuth/common/global"
	"errors"
	"gogs.baozhida.cn/zoie/OAuth-core/pkg/utils"
	"gogs.baozhida.cn/zoie/OAuth-core/service"

	"gorm.io/gorm"

	"Medical_OAuth/app/admin/model"
	"Medical_OAuth/app/admin/service/dto"
)

type SysRegisterAuth struct {
	service.Service
}

// GetPage 获取SysRegisterAuth列表
func (e *SysRegisterAuth) GetPage(c *dto.SysRegisterAuthPageReq, list *[]model.SysRegisterAuth, count *int64) error {
	var err error
	var data model.SysRegisterAuth

	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
		).
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("db error: %s \r", err)
		return global.GetFailedErr
	}
	return nil
}

// Get 获取SysRegisterAuth对象
func (e *SysRegisterAuth) Get(d *dto.SysRegisterAuthGetReq, postModel *model.SysRegisterAuth, p *actions.DataPermission) error {
	err := e.Orm.
		Scopes(actions.Permission(postModel.TableName(), p)).
		First(postModel, d.GetId()).Error

	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.GetNotFoundOrNoPermissionErr
		}
		return global.GetFailedErr
	}
	return nil
}

// Insert 创建SysRegisterAuth对象
func (e *SysRegisterAuth) Insert(c *dto.SysRegisterAuthInsertReq) error {
	var err error
	var data model.SysRegisterAuth

	var URKey string
	for {
		URKey = utils.GetRandString(16, "", 0)
		var j int64
		err = e.Orm.Model(&data).Where("ur_key = ?", URKey).Count(&j).Error
		if err != nil {
			continue
		}
		if j == 0 {
			break
		}
	}

	c.Generate(&data)
	data.URKey = URKey
	err = e.Orm.Create(&data).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.CreateFailedErr
	}
	c.Id = data.Id
	return nil
}

// Update 修改SysRegisterAuth对象
func (e *SysRegisterAuth) Update(c *dto.SysRegisterAuthUpdateReq) error {
	var postModel = model.SysRegisterAuth{}
	err := e.Orm.First(&postModel, c.GetId()).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return global.UpdateNotFoundOrNoPermissionErr
		}
		return global.UpdateFailedErr
	}

	c.Generate(&postModel)

	err = e.Orm.Save(&postModel).Error
	if err != nil {
		e.Log.Errorf("db error: %s", err)
		return global.UpdateFailedErr
	}
	return nil
}

// Remove 删除SysRegisterAuth
func (e *SysRegisterAuth) Remove(d *dto.SysRegisterAuthDeleteReq) error {
	var err error
	var data model.SysRegisterAuth

	db := e.Orm.Model(&data).Delete(&data, d.GetId())
	if err = db.Error; err != nil {
		e.Log.Errorf("Delete error: %s", err)
		return global.DeleteFailedErr
	}
	if db.RowsAffected == 0 {
		return global.DeleteNotFoundOrNoPermissionErr
	}
	return nil
}
